/*
     This file is part of gnunet-qt.
     (C) 2007 Nils Durner (and other contributing authors)

     gnunet-qt is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     gnunet-qt is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with gnunet-qt; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file src/plugins/fs/uploadController.cc
 * @brief Controller for *all* uploads
 * @author Nils Durner
 */

#include <math.h>
#include <QDir>
#include <QFileInfo>
#include <QStringList>
#include <QHeaderView>
#include <extractor.h>
#include <GNUnet/gnunet_ecrs_lib.h>
#include "uploadController.h"

static int getKeyword(const char *data, void *list)
{
  ((QStringList *) list)->append(QString::fromUtf8(data));
  
  return GNUNET_OK;
}

static int meta_data_get_by_type(EXTRACTOR_KeywordType type, const char *data, void *list)
{
  ((GFSMetaData *) list)->insert(type, data);
  
  return GNUNET_OK;
}

GFSUploadController::GFSUploadController(GFSPlugin *fs)
{
  this->fs = fs;
  extractors = NULL;
  
  uploadModel.setColumnCount(3);
  uploadModel.setHeaderData(0, Qt::Horizontal, tr("Filename"), Qt::DisplayRole);
  uploadModel.setHeaderData(1, Qt::Horizontal, tr("Progress"), Qt::DisplayRole);
  uploadModel.setHeaderData(2, Qt::Horizontal, tr("Status"), Qt::DisplayRole);
  
  fs->uploadView()->setModel(&uploadModel);
  fs->uploadView()->setItemDelegate(&delegate);
}

GFSUploadController::~GFSUploadController()
{
  if (extractors)
    EXTRACTOR_removeAll(extractors);
}

void GFSUploadController::setProgress(QPersistentModelIndex *idx,
  unsigned long long completed, unsigned long long total)
{
  QModelIndex item, parent;
  double progress;
  
  // compute progress for this entry
  progress = ((double) completed) / total * 100;
  if (isnan(progress))
    progress = 0;

  parent = idx->parent();
  item = uploadModel.index(idx->row(), 1, parent);
  uploadModel.setData(item, QVariant(progress), Qt::DisplayRole);
}

bool GFSUploadController::start(QWidget *parent, const QString &strPath,
  bool index, int prio, int anon)
{
  GFSMetaData metaData;
  QStringList keywords;
  struct GNUNET_ECRS_MetaData *meta;
  QFileInfo fileInfo;
  struct GNUNET_ECRS_URI *keywordUri;
  int thumbSize;
  unsigned char *thumb;

  parent->setCursor(Qt::WaitCursor);

  // load libextractor
  if (! extractors)
  {
    char *config;

    extractors = EXTRACTOR_loadDefaultLibraries();
    if (GNUNET_GC_get_configuration_value_string(fs->config(), "FS", "EXTRACTORS",
      NULL, &config) == 0 && config)
    {
      extractors = EXTRACTOR_loadConfigLibraries(extractors, config);
      GNUNET_free(config);
    }
  }

  // extract meta data
  meta = GNUNET_ECRS_meta_data_create();
  if (!meta)
  {
      GNUNET_GE_LOG(fs->errorContext(), (GNUNET_GE_KIND) (GNUNET_GE_ERROR | GNUNET_GE_USER | GNUNET_GE_IMMEDIATE),
        qPrintable(tr("Internal error: failed to create meta data for publication.")));
      return false;
  }
  
  GNUNET_ECRS_meta_data_extract_from_file(fs->errorContext(), meta, strPath.toLocal8Bit().data(), extractors);
  GNUNET_ECRS_meta_data_get_contents(meta, &meta_data_get_by_type, &metaData);
  thumbSize = GNUNET_ECRS_meta_data_get_thumbnail(meta, &thumb);
  if (thumbSize)
    metaData.insert(EXTRACTOR_THUMBNAIL_DATA, QByteArray((const char *) thumb,
      thumbSize));
  
  // Keywords
  keywordUri = GNUNET_ECRS_meta_data_to_uri(meta);
  GNUNET_ECRS_meta_data_destroy(meta);
  if (keywordUri)
  {
    GNUNET_ECRS_uri_get_keywords_from_ksk(keywordUri, getKeyword, &keywords);
    GNUNET_ECRS_uri_destroy(keywordUri);
  }

  fileInfo.setFile(strPath);
  if (fileInfo.isDir())
  {
    QDir dir(strPath);
    QString name = dir.dirName();

    if (!keywords.contains(name))    
      keywords.append(name);
  }  
 
  parent->setCursor(Qt::ArrowCursor);
 
  GFSUploadDialog uploadDlg(&metaData, &keywords, fs);
  
  if (uploadDlg.exec() == QDialog::Accepted)
  {
    struct GNUNET_FSUI_UploadList *handle;
    GFSMetaData::iterator it;
    struct GNUNET_ECRS_URI *globalURI, *keywordURI;
    char **keys;
    long idx;
    
    // prepare meta data
    meta = GNUNET_ECRS_meta_data_create();
    for(it = metaData.begin(); it != metaData.end(); it++)
      GNUNET_ECRS_meta_data_insert(meta, it.key(), it->data());

    // prepare global URI
    globalURI = GNUNET_ECRS_string_to_uri(fs->errorContext(), GNUNET_ECRS_URI_PREFIX
      GNUNET_ECRS_SEARCH_INFIX);
      
    // prepare keywords URI
    idx = keywords.count();
    keys = new char *[idx + 1];
    keys[idx] = NULL;
    idx--;
    while (idx >= 0)
    {
      keys[idx] = strdup(keywords[idx].toUtf8());
    
      idx--;
    }
    
    keywordURI = GNUNET_ECRS_keyword_strings_to_uri((const char **) keys);
    
    // upload
    handle = GNUNET_FSUI_upload_start(fs->context(), strPath.toLocal8Bit(),
      (GNUNET_FSUI_DirectoryScanCallback) &GNUNET_disk_directory_scan, fs->errorContext(),
      anon, prio, index, uploadDlg.extract(), uploadDlg.useKeywords(),
      GNUNET_get_time() + 2 * GNUNET_CRON_YEARS, meta, globalURI, keywordURI);
    
    // cleanup
    idx = 0;
    while(keys[idx])
    {
      free(keys[idx]);
      idx++;
    }
    delete [] keys;
    
    GNUNET_ECRS_meta_data_destroy(meta);
    GNUNET_ECRS_uri_destroy(globalURI);
    GNUNET_ECRS_uri_destroy(keywordURI);
    
    return true;
  }
  
  return false;
}

void GFSUploadController::state(QPersistentModelIndex *idx, GNUNET_FSUI_EventType event)
{
  QModelIndex index = uploadModel.index(idx->row(), 2, idx->parent());
  
  uploadModel.setData(index, QVariant(fs->fsuiState(event)), Qt::DisplayRole);
  uploadModel.setData(index, QVariant(event == GNUNET_FSUI_upload_completed),
    Qt::UserRole);
}

QAbstractItemModel *GFSUploadController::model()
{
  return &uploadModel;
}

QPersistentModelIndex *GFSUploadController::started(QPersistentModelIndex *parent,
    QString path, unsigned long long total, unsigned long long completed)
{
  QPersistentModelIndex *idx;
  QStandardItem *item, *parentItem;
  QString displayPath;
  
  displayPath = QFileInfo(path).fileName();
  if (displayPath == "")
    displayPath = QDir(path).dirName();
  
  if (parent)
    parentItem = uploadModel.itemFromIndex(uploadModel.index(parent->row(), parent->column(), parent->parent()));
  else
    parentItem = uploadModel.invisibleRootItem();
  
  item = new QStandardItem(displayPath);
  item->setColumnCount(3);
  parentItem->appendRow(item);
  
  idx = new QPersistentModelIndex(item->index());
  setProgress(idx, completed, total);

  state(idx, (total != completed || total == 0) ? GNUNET_FSUI_upload_started : GNUNET_FSUI_upload_completed);
  
  return idx;
}

void GFSUploadController::progress(QPersistentModelIndex *idx,
  unsigned long long completed, unsigned long long total)
{
  setProgress(idx, completed, total);
  state(idx, GNUNET_FSUI_upload_progress);
}

void GFSUploadController::clear()
{
  int row = 0;
  QModelIndex idx = uploadModel.index(0, 2);
  
  while(idx.isValid())
  {
    if (uploadModel.data(idx, Qt::UserRole).toInt() == 1)
      uploadModel.removeRow(row);
    else
      idx = uploadModel.index(++row, 2);
  }
}

/* end of uploadController.cc */
